@@ -1,4 +1,5 @@
 require 'puppet/version'
+require 'puppet/concurrent/synchronized'
 
 if Gem::Version.new(RUBY_VERSION.dup) < Gem::Version.new("2.3.0")
   raise LoadError, "Puppet #{Puppet.version} requires Ruby 2.3.0 or greater, found Ruby #{RUBY_VERSION.dup}."
@@ -6,6 +7,8 @@ end
 
 Puppet::OLDEST_RECOMMENDED_RUBY_VERSION = '2.3.0'
 
+$LOAD_PATH.extend(Puppet::Concurrent::Synchronized)
+
 # see the bottom of the file for further inclusions
 # Also see the new Vendor support - towards the end
 #
@@ -150,21 +153,45 @@ module Puppet
   # @param args [Array<String>] the command line arguments to use for initialization
   # @param require_config [Boolean] controls loading of Puppet configuration files
   # @param global_settings [Boolean] controls push to global context after settings object initialization
+  # @param runtime_implementations [Hash<Symbol, Object>] runtime implementations to register
   # @return [void]
-  def self.initialize_settings(args = [], require_config = true, push_settings_globally = true)
-    do_initialize_settings_for_run_mode(:user, args, require_config, push_settings_globally)
+  def self.initialize_settings(args = [], require_config = true, push_settings_globally = true, runtime_implementations = {})
+    do_initialize_settings_for_run_mode(:user, args, require_config, push_settings_globally, runtime_implementations)
   end
 
+  def self.vendored_modules
+    dir = Puppet[:vendormoduledir]
+    if dir && File.directory?(dir)
+      Dir.entries(dir)
+        .reject { |f| f =~ /^\./ }
+        .map { |f| File.join(dir, f, "lib") }
+        .select { |d| FileTest.directory?(d) }
+    else
+      []
+    end
+  end
+  private_class_method :vendored_modules
+
+  def self.initialize_load_path
+    $LOAD_PATH.unshift(Puppet[:libdir])
+    $LOAD_PATH.concat(vendored_modules)
+  end
+  private_class_method :initialize_load_path
+
   # private helper method to provide the implementation details of initializing for a run mode,
   #  but allowing us to control where the deprecation warning is issued
-  def self.do_initialize_settings_for_run_mode(run_mode, args, require_config = true, push_settings_globally = true)
+  def self.do_initialize_settings_for_run_mode(run_mode, args, require_config, push_settings_globally, runtime_implementations)
     Puppet.settings.initialize_global_settings(args, require_config)
     run_mode = Puppet::Util::RunMode[run_mode]
     Puppet.settings.initialize_app_defaults(Puppet::Settings.app_defaults_for_run_mode(run_mode))
     if push_settings_globally
+      initialize_load_path
       push_context_global(Puppet.base_context(Puppet.settings), "Initial context after settings initialization")
       Puppet::Parser::Functions.reset
     end
+    runtime_implementations.each_pair do |name, impl|
+      Puppet.runtime[name] = impl
+    end
   end
   private_class_method :do_initialize_settings_for_run_mode
 
@@ -218,10 +245,7 @@ module Puppet
 
     {
       :environments => Puppet::Environments::Cached.new(Puppet::Environments::Combined.new(*loaders)),
-      :http_pool => proc {
-        require 'puppet/network/http'
-        Puppet::Network::HTTP::NoCachePool.new
-      },
+      :http_pool => proc { Puppet.runtime[:http].pool },
       :ssl_context => proc {
         begin
           cert = Puppet::X509::CertProvider.new
@@ -236,8 +260,8 @@ module Puppet
           raise e
         end
       },
-      :ssl_host => proc { Puppet::SSL::Host.localhost },
-      :http_session => proc { Puppet.runtime["http"].create_session },
+      :ssl_host => proc { Puppet::SSL::Host.localhost(true) },
+      :http_session => proc { Puppet.runtime[:http].create_session },
       :plugins => proc { Puppet::Plugins::Configuration.load_plugins },
       :rich_data => false
     }
