@@ -64,8 +64,12 @@ module Puppet
       #
       # @return [void]
       def execute
+        require_config = true
+        if @argv.first =~ /help|-h|--help|-V|--version/
+          require_config = false
+        end
         Puppet::Util.exit_on_fail(_("Could not initialize global default settings")) do
-          Puppet.initialize_settings(args)
+          Puppet.initialize_settings(args, require_config)
         end
 
         setpriority(Puppet[:priority])
@@ -89,10 +93,13 @@ module Puppet
           end
         elsif Puppet::Application.available_application_names.include?(subcommand_name)
           ApplicationSubcommand.new(subcommand_name, self)
-        elsif path_to_subcommand = external_subcommand
-          ExternalSubcommand.new(path_to_subcommand, self)
         else
-          UnknownSubcommand.new(subcommand_name, self)
+          path_to_subcommand = external_subcommand
+          if path_to_subcommand
+            ExternalSubcommand.new(path_to_subcommand, self)
+          else
+            UnknownSubcommand.new(subcommand_name, self)
+          end
         end
       end
 
@@ -106,7 +113,7 @@ module Puppet
         def run
           # For most applications, we want to be able to load code from the modulepath,
           # such as apply, describe, resource, and faces.
-          # For agent, we only want to load pluginsync'ed code from libdir.
+          # For agent and device in agent mode, we only want to load pluginsync'ed code from libdir.
           # For master, we shouldn't ever be loading per-environment code into the master's
           # ruby process, but that requires fixing (#17210, #12173, #8750). So for now
           # we try to restrict to only code that can be autoloaded from the node's
@@ -116,9 +123,9 @@ module Puppet
           # have an appropriate application-wide current_environment set.
           # If we cannot find the configured environment, which may not exist,
           # we do not attempt to add plugin directories to the load path.
-          #
-          if @subcommand_name != 'master' and @subcommand_name != 'agent'
-            if configured_environment = Puppet.lookup(:environments).get(Puppet[:environment])
+          unless @subcommand_name == 'master' || @subcommand_name == 'agent' || (@subcommand_name == 'device' && (['--apply', '--facts', '--resource'] - @command_line.args).empty?)
+            configured_environment = Puppet.lookup(:environments).get(Puppet[:environment])
+            if configured_environment
               configured_environment.each_plugin_directory do |dir|
                 $LOAD_PATH << dir unless $LOAD_PATH.include?(dir)
               end
